

local LootLink = LootLink;




function LootLink:DataConversion_FromLootLink7xx_ToLootLink800(database)
	--[[
	The goal of this function is to convert all suffixes of each item to the new suffix format.
	Tables like this:
	
	[3] = {
		[suffix] = { suffixID, fullTextData };
	};
	
	become this:
	
	[3] = {
		[suffixID] = fullTextData;
	};
	
	The suffixID is then used to look up the actual suffix name from a static table. Once this format
	is established, expansion suffixes (which rely on instanceID to populate their stat values) can be
	added to make the table look like this:
	
	[3] = {
		[positive suffixID, indicating classic suffix] = fullTextData;
		[negative suffixID, indicating expansion suffix] = {
			[instanceID] = fullTextData;
		};
	};
	
	In the case that the user is not storing full text, or that there is none to store, the fullTextData
	value is replaced by the boolean true, as a placeholder to indicate validity of the key.
	]]--
	
	
	local pairs = pairs;
	local next = next;
	local type = type;
	local tonumber = tonumber;
	
	local currentTable;
	local newTable;
	local suffixID;
	
	
	for baseID, itemTable in pairs(database) do
		currentTable = itemTable[3];
		
		if (currentTable and type(next(currentTable)) == "string") then
			newTable = {};
			
			for suffixName, suffixTable in pairs(currentTable) do
				
				if (type(suffixName) == "string") then
					suffixID = tonumber(suffixTable[1]);
					
					if (suffixID > 0) then
						newTable[suffixID] = (suffixTable[2] or false);
					end
				end
				
			end
			
			itemTable[3] = newTable;
		end
	end
	
	
	collectgarbage("collect");
end




function LootLink:DataConversion_MergeTeloLootLink350_WithLootLinkCurrent(teloDatabase, database)
	local pairs = pairs;
	local type = type;
	local stringmatch = self.STRING_LIBRARY.match;
	local tonumber = tonumber;
	local tostring = tostring;
	local stringgmatch = self.STRING_LIBRARY.gmatch;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local hexToNumber = { ["ff9d9d9d"] = 0, ["ffffffff"] = 1, ["ff1eff00"] = 2, ["ff0070dd"] = 3, ["ffa335ee"] = 4, ["ff8000"] = 5, ["ffe6cc80"] = 6 };
	
	local baseID, suffixID;
	local currentTable;
	local baseName, suffixName;
	
	
	for itemName, itemTable in pairs(teloDatabase) do
		if (type(itemName) == "string" and itemTable.i) then
			
			baseID, suffixID = stringmatch(itemTable.i, "^(%-?%d+):%-?%d+:(%-?%d+):%-?%d+$");
			baseID = tonumber(baseID);
			
			if (baseID) then
				currentTable = database[baseID];
				
				if (not currentTable) then
					currentTable = {};
					database[baseID] = currentTable;
				end
				
				
				baseName, suffixName = stringmatch(itemName, "(.+) (of .+)");
				
				
				if (not currentTable[1]) then
					currentTable[1] = tostring(baseName or itemName);
				end
				
				
				if (not currentTable[2]) then
					currentTable[2] = (hexToNumber[itemTable.c] or 7);
				end
				
				
				suffixID = tonumber(suffixID);
				
				if (suffixID > 0 and suffixName) then
					if (not currentTable[3]) then
						currentTable[3] = {};
					end
					
					if (not currentTable[3][suffixID]) then
						currentTable[3][suffixID] = false;
					end
				end
				
				
				if (itemTable.t and not currentTable[5]) then
					currentTable[5] = {};
					
					for piece in stringgmatch(itemTable.t, "(.-)") do
						tableinsert(currentTable[5], piece);
					end
				end
				
				
				if (itemTable.p and not currentTable[6]) then
					currentTable[6] = tonumber(itemTable.p);
				end
			end
			
		end
		
		
		teloDatabase[itemName] = nil;
	end
	
	
	
	for baseID, itemTable in pairs(database) do
		
		if (type(baseID) ~= "number" or type(itemTable[1]) ~= "string") then
			database[baseID] = nil;
			
		else
			for key, value in pairs(itemTable) do
				if (type(key) ~= "number") then
					itemTable[key] = nil;
				end
			end
		end
		
	end
end




function LootLink:DataConversion_FromLootLink6xx_ToLootLink700(database, config)
	local stringgmatch = self.STRING_LIBRARY.gmatch;
	local stringmatch = self.STRING_LIBRARY.match;
	local stringsub = self.STRING_LIBRARY.sub;
	local tableinsert = self.TABLE_LIBRARY.insert;
	local pairs = pairs;
	
	local requiresPrefix = stringsub(REQUIRES_LABEL, 1, -2);
	local requiresLength = self.STRING_LIBRARY.len(requiresPrefix);
	
	local fullText, flagAdded;
	
	
	for baseID, itemTable in pairs(database) do
		-- To conserve memory, nil the "bind" key if it represents the "does not bind" type.
		if (itemTable[4] == 0) then itemTable[4] = nil; end
		
		
		local fullText = itemTable[5];
		
		if (fullText) then
			itemTable[5] = {};
			flagAdded = nil;
			
			for piece in stringgmatch(fullText, "(.-)") do
				if (not flagAdded and stringsub(piece, 1, requiresLength) == requiresPrefix and itemTable[3]) then
					tableinsert(itemTable[5], false);
					flagAdded = true;
				end
				tableinsert(itemTable[5], piece);
			end
		end
		
		
		if (itemTable[3]) then
			for suffix, suffixTable in pairs(itemTable[3]) do
				fullText = suffixTable[2];
				
				if (fullText) then
					suffixTable[2] = {};
					for piece in stringgmatch(fullText, "(.-)") do
						tableinsert(suffixTable[2], piece);
					end
				end
				
			end
		end
		
		
		-- Remove the faction, reputation, and rank keys, as well as all of the "last seen" keys.
		for index = 13, 50 do
			itemTable[index] = nil;
		end
	end
	
	
	config.ServerNumbersToNames = nil;
end




function LootLink:DataConversion_FromLootLink5xx_ToLootLink6xx(database, config)
	local stringsub = self.STRING_LIBRARY.sub;
	
	for key, value in pairs(config) do
		if (stringsub(key, -7) == "Channel" or stringsub(key, -8) == "Password") then
			config[key] = nil;
		end
	end
	
	
	local conversionTable = { bi = 4, lo = 7, ty = 8, su = 9, le = 10, un = 11, sk = 12, fa = 13, re = 14, ra = 15 };
	
	local tonumber = tonumber;
	local stringgmatch = self.STRING_LIBRARY.gmatch;
	
	local searchText;
	
	for baseID, itemTable in pairs(database) do
		-- Server numbers.
		for tag, value in stringgmatch(itemTable[7] or "", "(%d):(%d+)") do
			itemTable[tonumber(tag) + 15] = tonumber(value);
		end
		itemTable[7] = nil;
		
		-- Descriptor tags.
		searchText = itemTable[4];
		itemTable[4] = nil;
		for tag, value in stringgmatch(searchText or "", "([^%d]+)(%d+)") do
			if (conversionTable[tag]) then
				itemTable[conversionTable[tag]] = tonumber(value);
			end
		end
	end
end




function LootLink:DataConversion_MergeItemSync124_WithLootLink700(database, itemSyncDB, itemSyncNames)
	local stringmatch = self.STRING_LIBRARY.match;
	local pairs = pairs;
	local type = type;
	local tonumber = tonumber;
	
	local qualNum, sellPrice, level, subitemString, currentTable;
	
	local numNew = 0;
	
	
	for serverNum, serverTable in pairs(itemSyncDB) do
		for baseID, itemString in pairs(serverTable) do
			if (type(baseID) == "number" and itemSyncNames[baseID]) then
				
				qualNum, sellPrice, level, subitemString = stringmatch(itemString, "(%d+)(%d+)%d+%d+%d+%d+%d+(%d+)%d+%d+%d+(.+)");
				if (qualNum) then
					if (not database[baseID]) then
						database[baseID] = { [1] = itemSyncNames[baseID] };
					end
					numNew = (numNew + 1);
					currentTable = database[baseID];
					
					if (not currentTable[2]) then
						if (qualNum == "0") then
							currentTable[2] = 7; -- Unknown
						else
							currentTable[2] = (tonumber(qualNum) - 1);
						end
					end
					
					if (not currentTable[6]) then
						if (sellPrice ~= "0" and sellPrice ~= "1") then
							currentTable[6] = tonumber(sellPrice);
						end
					end
					
					if (not currentTable[10]) then
						if (level ~= "0") then
							currentTable[10] = tonumber(level);
						end
					end
					
					if (not currentTable[3]) then
						if (subitemString ~= "0") then
							currentTable[3] = {};
						end
					end
				end
				
			end
		end
	end
	
	
	return numNew;
end

function LootLink:DataConversion_MergeItemSync33816_WithLootLink829()
	local stringmatch = self.STRING_LIBRARY.match;
	local pairs = pairs;
  
  if not ItemSync then return end
  if ItemSync.version ~= 33816 then return end
  if not ISyncDB then return end
  if not ISyncDB["account"] then return end
	
  local suffixid, scaledSuffix

  linksTable = {}
  
  for serverName, serverTableData in pairs(ISyncDB["account"]) do
    for coreid, itemInfoString in pairs(serverTableData["items"]) do
      link = GetItemInfo(coreid)
      if link then
        --13th split equals suffixID and factor
        suffixid, scaledSuffix = stringmatch(itemInfoString, ".+.+.+.+.+.+.+.+.+.+.+.+(.+)(.+)")
        if suffixid and scaledSuffix then
          link = GetItemInfo("item:"..coreid..":0:0:0:0:0:"..suffixid..":"..scaledSuffix)
        end
        tableinsert(linksTable, link )
      else  --This item is not cached, add manually
        --quality,price,wl,wt,ts,at,st,level,vendor,vendorqty,idchk,resistances,subitem
        --[24846] = "20130030620000-633",
        quality, price, subdata = stringmatch(itemInfoString, "(.+)(.+).+.+.+.+.+.+.+.+.+.+(.+)")
        itemTable = self:ItemTables_AddBaseItem(coreid, baseName, quality)
        suffixid, scaledSuffix = stringmatch(subdata, "(.+)(.+)")
        suffixid = tonumber(suffixid)
        scaledSuffix = tonumber(scaledSuffix)
        if suffixid and suffixid ~= 0 then
          itemTable[3] = {[suffixid] = false}
          if scaledSuffix and scaledSuffix ~= 0 then
            itemTable[3][suffixid][scaledSuffix] = false
          end
        end
        itemTable[6] = price --vendor price in copper
--        itemTable[7] = 
--        itemTable[8] = 
--        itemTable[9] = 
      end
    end
  end
  
	if (#linksTable > 0) then
    LootLink:Parsing_ParseFromTable(linksTable)
  end
	
end
